@@ -12,6 +12,7 @@ from botocore.utils import parse_timestamp
 
 from tests import CLIRunner, AWSRequest, AWSResponse
 from awscli.testutils import unittest, FileCreator, mock
+from awscli.compat import urlparse
 from awscli.customizations.codeartifact.login import CodeArtifactLogin
 
 
@@ -43,6 +44,14 @@ class TestCodeArtifactLogin(unittest.TestCase):
         self.pypi_rc_path_mock = self.pypi_rc_path_patch.start()
         self.pypi_rc_path_mock.return_value = self.test_pypi_rc_path
 
+        self.test_netrc_path = self.file_creator.full_path('.netrc')
+        self.get_netrc_path_patch = mock.patch(
+            'awscli.customizations.codeartifact.login.SwiftLogin'
+            '.get_netrc_path'
+        )
+        self.get_netrc_mock = self.get_netrc_path_patch.start()
+        self.get_netrc_mock.return_value = self.test_netrc_path
+
         self.subprocess_patch = mock.patch('subprocess.run')
         self.subprocess_mock = self.subprocess_patch.start()
         self.subprocess_check_output_patch = mock.patch(
@@ -54,6 +63,8 @@ class TestCodeArtifactLogin(unittest.TestCase):
 
     def tearDown(self):
         self.pypi_rc_path_patch.stop()
+        self.subprocess_check_output_patch.stop()
+        self.get_netrc_path_patch.stop()
         self.subprocess_patch.stop()
         self.file_creator.remove_all()
 
@@ -109,6 +120,24 @@ class TestCodeArtifactLogin(unittest.TestCase):
 
         return cmdline
 
+    def _get_swift_commands(self, scope=None, token=None):
+        commands = []
+        set_registry_command = [
+            'swift', 'package-registry', 'set', self.endpoint
+        ]
+        if scope is not None:
+            set_registry_command.extend(['--scope', scope])
+        commands.append(set_registry_command)
+
+        login_registry_command = [
+            'swift', 'package-registry', 'login', f'{self.endpoint}login'
+        ]
+        if token is not None:
+            login_registry_command.extend(['--token', token])
+        commands.append(login_registry_command)
+
+        return commands
+
     def _get_nuget_commands(self):
         nuget_index_url = self.nuget_index_url_fmt.format(
             endpoint=self.endpoint
@@ -333,13 +362,183 @@ password: {auth_token}'''
             self.assertIn('password', pypi_rc.options(server))
             self.assertEqual(pypi_rc.get(server, 'password'), password)
 
+    def _assert_netrc_has_expected_content(self):
+        with open(self.test_netrc_path, 'r') as f:
+            actual_contents = f.read()
+
+        hostname = urlparse.urlparse(self.endpoint).hostname
+        expected_contents = f'machine {hostname} login token password {self.auth_token}\n'
+        self.assertEqual(expected_contents, actual_contents)
+
+    @mock.patch('awscli.customizations.codeartifact.login.is_macos', True)
+    def test_swift_login_without_domain_owner_macos(self):
+        cmdline = self._setup_cmd(tool='swift')
+        result = self.cli_runner.run(cmdline)
+        self.assertEqual(result.rc, 0)
+        self._assert_operations_called(package_format='swift', result=result)
+        self._assert_expiration_printed_to_stdout(result.stdout)
+        self._assert_subprocess_execution(
+            self._get_swift_commands(token=self.auth_token)
+        )
+        self.assertFalse(os.path.exists(self.test_netrc_path))
+
+    @mock.patch('awscli.customizations.codeartifact.login.is_macos', False)
+    def test_swift_login_without_domain_owner_non_macos(self):
+        cmdline = self._setup_cmd(tool='swift')
+        result = self.cli_runner.run(cmdline)
+        self.assertEqual(result.rc, 0)
+        self._assert_operations_called(package_format='swift', result=result)
+        self._assert_expiration_printed_to_stdout(result.stdout)
+        self._assert_subprocess_execution(
+            self._get_swift_commands()
+        )
+        self._assert_netrc_has_expected_content()
+
+    @mock.patch('awscli.customizations.codeartifact.login.is_macos', False)
+    def test_swift_login_without_domain_owner_dry_run(self):
+        cmdline = self._setup_cmd(tool='swift', dry_run=True)
+        result = self.cli_runner.run(cmdline)
+        self.assertEqual(result.rc, 0)
+        self._assert_operations_called(package_format='swift', result=result)
+        self._assert_dry_run_execution(self._get_swift_commands(), result.stdout)
+        self.assertFalse(os.path.exists(self.test_netrc_path))
+
+    @mock.patch('awscli.customizations.codeartifact.login.is_macos', True)
+    def test_swift_login_with_domain_owner_macos(self):
+        cmdline = self._setup_cmd(tool='swift', include_domain_owner=True)
+        result = self.cli_runner.run(cmdline)
+        self.assertEqual(result.rc, 0)
+        self._assert_operations_called(
+            package_format='swift', result=result,
+            include_domain_owner=True, include_duration_seconds=False
+        )
+        self._assert_expiration_printed_to_stdout(result.stdout)
+        self._assert_subprocess_execution(
+            self._get_swift_commands(token=self.auth_token)
+        )
+
+    @mock.patch('awscli.customizations.codeartifact.login.is_macos', False)
+    def test_swift_login_with_domain_owner_non_macos(self):
+        cmdline = self._setup_cmd(tool='swift', include_domain_owner=True)
+        result = self.cli_runner.run(cmdline)
+        self.assertEqual(result.rc, 0)
+        self._assert_operations_called(
+            package_format='swift', result=result,
+            include_domain_owner=True, include_duration_seconds=False
+        )
+        self._assert_expiration_printed_to_stdout(result.stdout)
+        self._assert_subprocess_execution(
+            self._get_swift_commands()
+        )
+        self._assert_netrc_has_expected_content()
+
+    @mock.patch('awscli.customizations.codeartifact.login.is_macos', True)
+    def test_swift_login_with_domain_owner_duration_macos(self):
+        cmdline = self._setup_cmd(tool='swift', include_domain_owner=True,
+                                  include_duration_seconds=True)
+        result = self.cli_runner.run(cmdline)
+        self.assertEqual(result.rc, 0)
+        self._assert_operations_called(
+            package_format='swift', result=result,
+            include_domain_owner=True, include_duration_seconds=True
+        )
+        self._assert_expiration_printed_to_stdout(result.stdout)
+        self._assert_subprocess_execution(
+            self._get_swift_commands(token=self.auth_token)
+        )
+
+    @mock.patch('awscli.customizations.codeartifact.login.is_macos', False)
+    def test_swift_login_with_domain_owner_duration_non_macos(self):
+        cmdline = self._setup_cmd(tool='swift', include_domain_owner=True,
+                                  include_duration_seconds=True)
+        result = self.cli_runner.run(cmdline)
+        self.assertEqual(result.rc, 0)
+        self._assert_operations_called(
+            package_format='swift', result=result,
+            include_domain_owner=True, include_duration_seconds=True
+        )
+        self._assert_expiration_printed_to_stdout(result.stdout)
+        self._assert_subprocess_execution(
+            self._get_swift_commands()
+        )
+        self._assert_netrc_has_expected_content()
+
+    @mock.patch('awscli.customizations.codeartifact.login.is_macos', False)
+    def test_swift_login_with_domain_owner_dry_run(self):
+        cmdline = self._setup_cmd(
+            tool='swift', include_domain_owner=True, dry_run=True
+        )
+        result = self.cli_runner.run(cmdline)
+        self.assertEqual(result.rc, 0)
+        self._assert_operations_called(
+            package_format='swift', result=result, include_domain_owner=True
+        )
+        self._assert_dry_run_execution(
+            self._get_swift_commands(), result.stdout
+        )
+        self.assertFalse(os.path.exists(self.test_netrc_path))
+
+    @mock.patch('awscli.customizations.codeartifact.login.is_macos', True)
+    def test_swift_login_with_domain_owner_duration_dry_run(self):
+        cmdline = self._setup_cmd(
+            tool='swift', include_domain_owner=True,
+            include_duration_seconds=True, dry_run=True
+        )
+        result = self.cli_runner.run(cmdline)
+        self.assertEqual(result.rc, 0)
+        self._assert_operations_called(
+            package_format='swift', result=result, include_domain_owner=True,
+            include_duration_seconds=True
+        )
+        self._assert_dry_run_execution(
+            self._get_swift_commands(token=self.auth_token), result.stdout
+        )
+
+    @mock.patch('awscli.customizations.codeartifact.login.is_macos', True)
+    def test_swift_login_with_namespace_macos(self):
+        cmdline = self._setup_cmd(
+            tool='swift', include_namespace=True
+        )
+        result = self.cli_runner.run(cmdline)
+        self.assertEqual(result.rc, 0)
+        self._assert_operations_called(package_format='swift', result=result)
+        self._assert_expiration_printed_to_stdout(result.stdout)
+        self._assert_subprocess_execution(
+            self._get_swift_commands(scope=self.namespace, token=self.auth_token)
+        )
+
+    @mock.patch('awscli.customizations.codeartifact.login.is_macos', False)
+    def test_swift_login_with_namespace_non_macos(self):
+        cmdline = self._setup_cmd(
+            tool='swift', include_namespace=True
+        )
+        result = self.cli_runner.run(cmdline)
+        self.assertEqual(result.rc, 0)
+        self._assert_operations_called(package_format='swift', result=result)
+        self._assert_expiration_printed_to_stdout(result.stdout)
+        self._assert_subprocess_execution(
+            self._get_swift_commands(scope=self.namespace)
+        )
+        self._assert_netrc_has_expected_content()
+
+    @mock.patch('awscli.customizations.codeartifact.login.is_macos', True)
+    def test_swift_login_with_namespace_dry_run(self):
+        cmdline = self._setup_cmd(
+            tool='swift', include_namespace=True, dry_run=True
+        )
+        result = self.cli_runner.run(cmdline)
+        self.assertEqual(result.rc, 0)
+        self._assert_operations_called(package_format='swift', result=result)
+        self._assert_dry_run_execution(
+            self._get_swift_commands(scope=self.namespace),result.stdout)
+
     def test_nuget_login_without_domain_owner_without_duration_seconds(self):
         cmdline = self._setup_cmd(tool='nuget')
         result = self.cli_runner.run(cmdline)
         self.assertEqual(result.rc, 0)
         self._assert_operations_called(package_format='nuget', result=result)
         self._assert_expiration_printed_to_stdout(result.stdout)
-        self._assert_subprocess_check_output_execution(
+        self._assert_subprocess_execution(
             self._get_nuget_commands()
         )
 
@@ -353,7 +552,7 @@ password: {auth_token}'''
             result=result
         )
         self._assert_expiration_printed_to_stdout(result.stdout)
-        self._assert_subprocess_check_output_execution(
+        self._assert_subprocess_execution(
             self._get_nuget_commands()
         )
 
@@ -367,7 +566,7 @@ password: {auth_token}'''
             result=result
         )
         self._assert_expiration_printed_to_stdout(result.stdout)
-        self._assert_subprocess_check_output_execution(
+        self._assert_subprocess_execution(
             self._get_nuget_commands()
         )
 
@@ -386,7 +585,7 @@ password: {auth_token}'''
             result=result
         )
         self._assert_expiration_printed_to_stdout(result.stdout)
-        self._assert_subprocess_check_output_execution(
+        self._assert_subprocess_execution(
             self._get_nuget_commands()
         )
 
@@ -457,7 +656,7 @@ password: {auth_token}'''
         self.assertEqual(result.rc, 0)
         self._assert_operations_called(package_format='nuget', result=result)
         self._assert_expiration_printed_to_stdout(result.stdout)
-        self._assert_subprocess_check_output_execution(
+        self._assert_subprocess_execution(
             self._get_dotnet_commands()
         )
 
@@ -472,7 +671,7 @@ password: {auth_token}'''
             result=result
         )
         self._assert_expiration_printed_to_stdout(result.stdout)
-        self._assert_subprocess_check_output_execution(
+        self._assert_subprocess_execution(
             self._get_dotnet_commands()
         )
 
@@ -487,7 +686,7 @@ password: {auth_token}'''
             result=result
         )
         self._assert_expiration_printed_to_stdout(result.stdout)
-        self._assert_subprocess_check_output_execution(
+        self._assert_subprocess_execution(
             self._get_dotnet_commands()
         )
 
@@ -507,7 +706,7 @@ password: {auth_token}'''
             result=result
         )
         self._assert_expiration_printed_to_stdout(result.stdout)
-        self._assert_subprocess_check_output_execution(
+        self._assert_subprocess_execution(
             self._get_dotnet_commands()
         )
 
