@@ -47,7 +47,7 @@ COMMANDS = [
     'kinesis list-streams',
     'kms generate-random --number-of-bytes 128',
     'logs describe-log-groups',
-    'opsworks describe-stacks',
+    # 'opsworks describe-stacks',
     'rds describe-db-instances',
     'redshift describe-clusters',
     'route53 list-hosted-zones',
@@ -94,7 +94,7 @@ ERROR_COMMANDS = [
     'iam delete-user --user-name %s',
     'kinesis delete-stream --stream-name %s',
     'logs delete-log-group --log-group-name %s',
-    'opsworks delete-app --app-id %s',
+    # 'opsworks delete-app --app-id %s',
     'rds delete-db-instance --db-instance-identifier %s',
     'redshift delete-cluster --cluster-identifier %s',
     'route53 delete-hosted-zone --id %s',
