@@ -24,14 +24,31 @@ from botocore.config import Config
 from botocore.configprovider import ConfigValueStore
 from botocore.hooks import HierarchicalEmitter
 from botocore.model import ServiceModel
+from botocore.useragent import UserAgentString
 
 
 class TestCreateClientArgs(unittest.TestCase):
     def setUp(self):
         self.event_emitter = mock.Mock(HierarchicalEmitter)
         self.config_store = ConfigValueStore()
+        user_agent_creator = UserAgentString(
+            platform_name=None,
+            platform_version=None,
+            platform_machine=None,
+            python_version=None,
+            python_implementation=None,
+            execution_env=None,
+            crt_version=None,
+        )
         self.args_create = args.ClientArgsCreator(
-            self.event_emitter, None, None, None, None, self.config_store)
+            event_emitter=self.event_emitter,
+            user_agent=None,
+            response_parser_factory=None,
+            loader=None,
+            exceptions_factory=None,
+            config_store=self.config_store,
+            user_agent_creator=user_agent_creator,
+        )
         self.service_name = 'ec2'
         self.region = 'us-west-2'
         self.endpoint_url = 'https://ec2/'
@@ -349,11 +366,105 @@ class TestCreateClientArgs(unittest.TestCase):
             )
             m.assert_not_called()
 
+    def test_request_compression_client_config(self):
+        input_config = Config(
+            disable_request_compression=True,
+            request_min_compression_size_bytes=100,
+        )
+        client_args = self.call_get_client_args(client_config=input_config)
+        config = client_args['client_config']
+        self.assertEqual(config.request_min_compression_size_bytes, 100)
+        self.assertTrue(config.disable_request_compression)
+
+    def test_request_compression_config_store(self):
+        self.config_store.set_config_variable(
+            'request_min_compression_size_bytes', 100
+        )
+        self.config_store.set_config_variable(
+            'disable_request_compression', True
+        )
+        config = self.call_get_client_args()['client_config']
+        self.assertEqual(config.request_min_compression_size_bytes, 100)
+        self.assertTrue(config.disable_request_compression)
+
+    def test_request_compression_client_config_overrides_config_store(self):
+        self.config_store.set_config_variable(
+            'request_min_compression_size_bytes', 100
+        )
+        self.config_store.set_config_variable(
+            'disable_request_compression', True
+        )
+        input_config = Config(
+            disable_request_compression=False,
+            request_min_compression_size_bytes=1,
+        )
+        client_args = self.call_get_client_args(client_config=input_config)
+        config = client_args['client_config']
+        self.assertEqual(config.request_min_compression_size_bytes, 1)
+        self.assertFalse(config.disable_request_compression)
+
+    def test_coercible_value_request_min_compression_size_bytes(self):
+        config = Config(request_min_compression_size_bytes='100')
+        client_args = self.call_get_client_args(client_config=config)
+        config = client_args['client_config']
+        self.assertEqual(config.request_min_compression_size_bytes, 100)
+
+    def test_coercible_value_disable_request_compression(self):
+        config = Config(disable_request_compression='true')
+        client_args = self.call_get_client_args(client_config=config)
+        config = client_args['client_config']
+        self.assertTrue(config.disable_request_compression)
+
+    def test_bad_type_request_min_compression_size_bytes(self):
+        with self.assertRaises(exceptions.InvalidConfigError):
+            config = Config(request_min_compression_size_bytes='foo')
+            self.call_get_client_args(client_config=config)
+        self.config_store.set_config_variable(
+            'request_min_compression_size_bytes', 'foo'
+        )
+        with self.assertRaises(exceptions.InvalidConfigError):
+            self.call_get_client_args()
+
+    def test_low_min_request_min_compression_size_bytes(self):
+        with self.assertRaises(exceptions.InvalidConfigError):
+            config = Config(request_min_compression_size_bytes=0)
+            self.call_get_client_args(client_config=config)
+        self.config_store.set_config_variable(
+            'request_min_compression_size_bytes', 0
+        )
+        with self.assertRaises(exceptions.InvalidConfigError):
+            self.call_get_client_args()
+
+    def test_high_max_request_min_compression_size_bytes(self):
+        with self.assertRaises(exceptions.InvalidConfigError):
+            config = Config(request_min_compression_size_bytes=9999999)
+            self.call_get_client_args(client_config=config)
+        self.config_store.set_config_variable(
+            'request_min_compression_size_bytes', 9999999
+        )
+        with self.assertRaises(exceptions.InvalidConfigError):
+            self.call_get_client_args()
+
+    def test_bad_value_disable_request_compression(self):
+        input_config = Config(disable_request_compression='foo')
+        client_args = self.call_get_client_args(client_config=input_config)
+        config = client_args['client_config']
+        self.assertFalse(config.disable_request_compression)
+
 
 class TestEndpointResolverBuiltins(unittest.TestCase):
     def setUp(self):
         event_emitter = mock.Mock(HierarchicalEmitter)
         self.config_store = ConfigValueStore()
+        user_agent_creator = UserAgentString(
+            platform_name=None,
+            platform_version=None,
+            platform_machine=None,
+            python_version=None,
+            python_implementation=None,
+            execution_env=None,
+            crt_version=None,
+        )
         self.args_create = args.ClientArgsCreator(
             event_emitter=event_emitter,
             user_agent=None,
@@ -361,6 +472,7 @@ class TestEndpointResolverBuiltins(unittest.TestCase):
             loader=None,
             exceptions_factory=None,
             config_store=self.config_store,
+            user_agent_creator=user_agent_creator,
         )
         self.bridge = ClientEndpointBridge(
             endpoint_resolver=mock.Mock(),
