@@ -3,14 +3,15 @@ import botocore.session
 
 from mock import patch, Mock, MagicMock
 from botocore.stub import Stubber
-from awscli.testutils import unittest
 from awscli.customizations.cloudformation.deployer import Deployer, ChangeSetResult
 from awscli.customizations.cloudformation import exceptions
+from tests.unit.customizations.cloudformation import BaseYAMLTest
 
 
-class TestDeployer(unittest.TestCase):
+class TestDeployer(BaseYAMLTest):
 
     def setUp(self):
+        super(TestDeployer, self).setUp()
         client = botocore.session.get_session().create_client('cloudformation',
                                                               region_name="us-east-1")
         self.stub_client = Stubber(client)
@@ -263,13 +264,30 @@ class TestDeployer(unittest.TestCase):
 
         expected_params = {
             "ChangeSetName": changeset_id,
-            "StackName": stack_name
+            "StackName": stack_name,
+            "DisableRollback": False
         }
 
         self.stub_client.add_response("execute_change_set", {}, expected_params)
         with self.stub_client:
             self.deployer.execute_changeset(changeset_id, stack_name)
 
+    def test_execute_changeset_disable_rollback(self):
+        stack_name = "stack_name"
+        changeset_id = "changeset_id"
+        disable_rollback = True
+
+        expected_params = {
+            "ChangeSetName": changeset_id,
+            "StackName": stack_name,
+            "DisableRollback": disable_rollback
+        }
+
+        self.stub_client.add_response("execute_change_set", {}, expected_params)
+        with self.stub_client:
+            self.deployer.execute_changeset(changeset_id, stack_name,
+                                            disable_rollback)
+
     def test_execute_changeset_exception(self):
         stack_name = "stack_name"
         changeset_id = "changeset_id"
