@@ -208,11 +208,11 @@ class TestAddSteps(BaseAWSCommandParamsTest):
 
     def test_custom_jar_step_with_all_fields(self):
         cmd = self.prefix + (
-            'Name=Custom,Type=Custom_jar,'
-            'Jar=s3://mybucket/mytest.jar,'
-            'Args=arg1,arg2,MainClass=mymainclass,'
-            'ActionOnFailure=TERMINATE_CLUSTER,'
-            'Properties=k1=v1\,k2=v2\,k3')
+            r'Name=Custom,Type=Custom_jar,'
+            r'Jar=s3://mybucket/mytest.jar,'
+            r'Args=arg1,arg2,MainClass=mymainclass,'
+            r'ActionOnFailure=TERMINATE_CLUSTER,'
+            r'Properties=k1=v1\,k2=v2\,k3')
         expected_result = {
             'JobFlowId': 'j-ABC',
             'Steps': [
@@ -255,6 +255,27 @@ class TestAddSteps(BaseAWSCommandParamsTest):
             cmd=cmd, expected_result=expected_result,
             expected_result_release=expected_result_release)
 
+    def test_step_with_execution_role_arn(self):
+        cmd = self.prefix + 'Type=Streaming,' + self.STREAMING_ARGS
+        cmd += ' --execution-role-arn arn:aws:iam::123456789010:role/sample '
+        expected_result = {
+            'ExecutionRoleArn': 'arn:aws:iam::123456789010:role/sample',
+            'JobFlowId': 'j-ABC',
+            'Steps': [
+                {'Name': 'Streaming program',
+                 'ActionOnFailure': 'CONTINUE',
+                 'HadoopJarStep': self.STREAMING_HADOOP_SCRIPT_RUNNER_STEP
+                 }
+            ]
+        }
+        expected_result_release = copy.deepcopy(expected_result)
+        expected_result_release['Steps'][0]['HadoopJarStep'] = \
+            self.STREAMING_HADOOP_COMMAND_RUNNER_STEP
+
+        self.assert_params_for_ami_and_release_based_clusters(
+            cmd=cmd, expected_result=expected_result,
+            expected_result_release=expected_result_release)
+
     def test_streaming_step_missing_args(self):
         cmd = self.prefix + 'Type=Streaming'
         expected_error_msg = '\naws: error: The following ' + \
@@ -629,11 +650,11 @@ class TestAddSteps(BaseAWSCommandParamsTest):
             expected_result_release):
         if expected_error_msg:
             grl_patch.return_value = None
-            result = self.run_cmd(cmd, 255)
+            result = self.run_cmd(cmd, 252)
             self.assertEqual(expected_error_msg, result[1])
         if expected_result_release:
             grl_patch.return_value = 'emr-4.0'
-            result = self.run_cmd(cmd, 255)
+            result = self.run_cmd(cmd, 252)
             self.assertEqual(expected_result_release, result[1])
 
 if __name__ == "__main__":
