@@ -12,82 +12,98 @@
 # language governing permissions and limitations under the License.
 from awscli.testutils import unittest, temporary_file
 import argparse
-import errno
 import os
-import tempfile
-import shutil
+
 import ntpath
 import time
 import datetime
 
 import mock
 from dateutil.tz import tzlocal
-from nose.tools import assert_equal
-from s3transfer.futures import TransferMeta, TransferFuture
+import pytest
 from s3transfer.compat import seekable
 from botocore.hooks import HierarchicalEmitter
 
-from awscli.compat import queue
 from awscli.compat import StringIO
-from awscli.testutils import FileCreator
+from awscli.customizations.exceptions import ParamValidationError
 from awscli.customizations.s3.utils import (
     find_bucket_key,
     guess_content_type, relative_path, block_unsupported_resources,
     StablePriorityQueue, BucketLister, get_file_stat, AppendFilter,
-    create_warning, human_readable_size, human_readable_to_bytes,
+    create_warning, human_readable_size, human_readable_to_int,
     set_file_utime, SetFileUtimeError, RequestParamsMapper, StdoutBytesWriter,
-    ProvideSizeSubscriber, OnDoneFilteredSubscriber,
-    ProvideUploadContentTypeSubscriber, ProvideCopyContentTypeSubscriber,
-    ProvideLastModifiedTimeSubscriber, DirectoryCreatorSubscriber,
-    DeleteSourceObjectSubscriber, DeleteSourceFileSubscriber,
-    DeleteCopySourceObjectSubscriber, NonSeekableStream, CreateDirectoryError)
-from awscli.customizations.s3.results import WarningResult
-from tests.unit.customizations.s3 import FakeTransferFuture
-from tests.unit.customizations.s3 import FakeTransferFutureMeta
-from tests.unit.customizations.s3 import FakeTransferFutureCallArgs
+    NonSeekableStream, S3PathResolver
+)
 
 
-def test_human_readable_size():
-    yield _test_human_size_matches, 1, '1 Byte'
-    yield _test_human_size_matches, 10, '10 Bytes'
-    yield _test_human_size_matches, 1000, '1000 Bytes'
-    yield _test_human_size_matches, 1024, '1.0 KiB'
-    yield _test_human_size_matches, 1024 ** 2, '1.0 MiB'
-    yield _test_human_size_matches, 1024 ** 2, '1.0 MiB'
-    yield _test_human_size_matches, 1024 ** 3, '1.0 GiB'
-    yield _test_human_size_matches, 1024 ** 4, '1.0 TiB'
-    yield _test_human_size_matches, 1024 ** 5, '1.0 PiB'
-    yield _test_human_size_matches, 1024 ** 6, '1.0 EiB'
-
-    # Round to the nearest block.
-    yield _test_human_size_matches, 1024 ** 2 - 1, '1.0 MiB'
-    yield _test_human_size_matches, 1024 ** 3 - 1, '1.0 GiB'
+@pytest.fixture
+def s3control_client():
+    client = mock.MagicMock()
+    client.get_access_point.return_value = {
+        "Bucket": "mybucket"
+    }
+    client.list_multi_region_access_points.return_value = {
+        "AccessPoints": [{
+            "Alias": "myalias.mrap",
+            "Regions": [{"Bucket": "mybucket"}]
+        }]
+    }
+    return client
 
 
-def _test_human_size_matches(bytes_int, expected):
-    assert_equal(human_readable_size(bytes_int), expected)
+@pytest.fixture
+def sts_client():
+    client = mock.MagicMock()
+    client.get_caller_identity.return_value = {
+        "Account": "123456789012"
+    }
+    return client
 
 
-def test_convert_human_readable_to_bytes():
-    yield _test_convert_human_readable_to_bytes, "1", 1
-    yield _test_convert_human_readable_to_bytes, "1024", 1024
-    yield _test_convert_human_readable_to_bytes, "1KB", 1024
-    yield _test_convert_human_readable_to_bytes, "1kb", 1024
-    yield _test_convert_human_readable_to_bytes, "1MB", 1024 ** 2
-    yield _test_convert_human_readable_to_bytes, "1GB", 1024 ** 3
-    yield _test_convert_human_readable_to_bytes, "1TB", 1024 ** 4
-
-    # Also because of the "ls" output for s3, we support
-    # the IEC "mebibyte" format (MiB).
-    yield _test_convert_human_readable_to_bytes, "1KiB", 1024
-    yield _test_convert_human_readable_to_bytes, "1kib", 1024
-    yield _test_convert_human_readable_to_bytes, "1MiB", 1024 ** 2
-    yield _test_convert_human_readable_to_bytes, "1GiB", 1024 ** 3
-    yield _test_convert_human_readable_to_bytes, "1TiB", 1024 ** 4
+@pytest.fixture
+def s3_path_resolver(s3control_client, sts_client):
+    return S3PathResolver(s3control_client, sts_client)
 
 
-def _test_convert_human_readable_to_bytes(size_str, expected):
-    assert_equal(human_readable_to_bytes(size_str), expected)
+@pytest.mark.parametrize(
+    'bytes_int, expected',
+    (
+        (1, '1 Byte'),
+        (10, '10 Bytes'),
+        (1000, '1000 Bytes'),
+        (1024, '1.0 KiB'),
+        (1024 ** 2, '1.0 MiB'),
+        (1024 ** 3, '1.0 GiB'),
+        (1024 ** 4, '1.0 TiB'),
+        (1024 ** 5, '1.0 PiB'),
+        (1024 ** 6, '1.0 EiB'),
+        (1024 ** 2 - 1, '1.0 MiB'),
+        (1024 ** 3 - 1, '1.0 GiB'),
+    )
+)
+def test_human_readable_size(bytes_int, expected):
+    assert human_readable_size(bytes_int) == expected
+
+
+@pytest.mark.parametrize(
+    'size_str, expected',
+    (
+        ("1", 1),
+        ("1024", 1024),
+        ("1KB", 1024),
+        ("1kb", 1024),
+        ("1MB", 1024 ** 2),
+        ("1GB", 1024 ** 3),
+        ("1TB", 1024 ** 4),
+        ("1KiB", 1024),
+        ("1kib", 1024),
+        ("1MiB", 1024 ** 2),
+        ("1GiB", 1024 ** 3),
+        ("1TiB", 1024 ** 4),
+    )
+)
+def test_convert_human_readable_to_int(size_str, expected):
+    assert human_readable_to_int(size_str) == expected
 
 
 class AppendFilterTest(unittest.TestCase):
@@ -302,7 +318,7 @@ class TestFindBucketKey(unittest.TestCase):
 class TestBlockUnsupportedResources(unittest.TestCase):
     def test_object_lambda_arn_with_colon_raises_exception(self):
         with self.assertRaisesRegex(
-                ValueError, 'Use s3api commands instead'):
+                ParamValidationError, 'Use s3api commands instead'):
             block_unsupported_resources(
                 'arn:aws:s3-object-lambda:us-west-2:123456789012:'
                 'accesspoint:my-accesspoint'
@@ -310,7 +326,7 @@ class TestBlockUnsupportedResources(unittest.TestCase):
 
     def test_object_lambda_arn_with_slash_raises_exception(self):
         with self.assertRaisesRegex(
-                ValueError, 'Use s3api commands instead'):
+                ParamValidationError, 'Use s3api commands instead'):
             block_unsupported_resources(
                  'arn:aws:s3-object-lambda:us-west-2:123456789012:'
                  'accesspoint/my-accesspoint'
@@ -318,7 +334,7 @@ class TestBlockUnsupportedResources(unittest.TestCase):
 
     def test_outpost_bucket_arn_with_colon_raises_exception(self):
         with self.assertRaisesRegex(
-                ValueError, 'Use s3control commands instead'):
+                ParamValidationError, 'Use s3control commands instead'):
             block_unsupported_resources(
                 'arn:aws:s3-outposts:us-west-2:123456789012:'
                 'outpost/op-0a12345678abcdefg:bucket/bucket-foo'
@@ -326,7 +342,7 @@ class TestBlockUnsupportedResources(unittest.TestCase):
 
     def test_outpost_bucket_arn_with_slash_raises_exception(self):
         with self.assertRaisesRegex(
-                ValueError, 'Use s3control commands instead'):
+                ParamValidationError, 'Use s3control commands instead'):
             block_unsupported_resources(
                  'arn:aws:s3-outposts:us-west-2:123456789012:'
                  'outpost/op-0a12345678abcdefg/bucket/bucket-foo'
@@ -502,7 +518,7 @@ class TestGetFileStat(unittest.TestCase):
 
     def test_error_message(self):
         with mock.patch('os.stat', mock.Mock(side_effect=IOError('msg'))):
-            with self.assertRaisesRegex(ValueError, 'myfilename\.txt'):
+            with self.assertRaisesRegex(ValueError, r'myfilename\.txt'):
                 get_file_stat('myfilename.txt')
 
     def assert_handles_fromtimestamp_error(self, error):
@@ -691,6 +707,18 @@ class TestRequestParamsMapperRequestPayer(unittest.TestCase):
             params, self.cli_params)
         self.assertEqual(params, {'RequestPayer': 'requester'})
 
+    def test_map_get_object_tagging_params(self):
+        params = {}
+        RequestParamsMapper.map_get_object_tagging_params(
+            params, self.cli_params)
+        self.assertEqual(params, {'RequestPayer': 'requester'})
+
+    def test_map_put_object_tagging_params(self):
+        params = {}
+        RequestParamsMapper.map_put_object_tagging_params(
+            params, self.cli_params)
+        self.assertEqual(params, {'RequestPayer': 'requester'})
+
 
 class TestBytesPrint(unittest.TestCase):
     def setUp(self):
@@ -704,270 +732,6 @@ class TestBytesPrint(unittest.TestCase):
         self.assertEqual(self.stdout.write.call_args[0][0], b'foo')
 
 
-class TestProvideSizeSubscriber(unittest.TestCase):
-    def setUp(self):
-        self.transfer_future = mock.Mock(spec=TransferFuture)
-        self.transfer_meta = TransferMeta()
-        self.transfer_future.meta = self.transfer_meta
-
-    def test_size_set(self):
-        self.transfer_meta.provide_transfer_size(5)
-        subscriber = ProvideSizeSubscriber(10)
-        subscriber.on_queued(self.transfer_future)
-        self.assertEqual(self.transfer_meta.size, 10)
-
-
-class OnDoneFilteredRecordingSubscriber(OnDoneFilteredSubscriber):
-    def __init__(self):
-        self.on_success_calls = []
-        self.on_failure_calls = []
-
-    def _on_success(self, future):
-        self.on_success_calls.append(future)
-
-    def _on_failure(self, future, exception):
-        self.on_failure_calls.append((future, exception))
-
-
-class TestOnDoneFilteredSubscriber(unittest.TestCase):
-    def test_on_success(self):
-        subscriber = OnDoneFilteredRecordingSubscriber()
-        future = FakeTransferFuture('return-value')
-        subscriber.on_done(future)
-        self.assertEqual(subscriber.on_success_calls, [future])
-        self.assertEqual(subscriber.on_failure_calls, [])
-
-    def test_on_failure(self):
-        subscriber = OnDoneFilteredRecordingSubscriber()
-        exception = Exception('my exception')
-        future = FakeTransferFuture(exception=exception)
-        subscriber.on_done(future)
-        self.assertEqual(subscriber.on_failure_calls, [(future, exception)])
-        self.assertEqual(subscriber.on_success_calls, [])
-
-
-class TestProvideUploadContentTypeSubscriber(unittest.TestCase):
-    def setUp(self):
-        self.filename = 'myfile.txt'
-        self.extra_args = {}
-        self.future = self.set_future()
-        self.subscriber = ProvideUploadContentTypeSubscriber()
-
-    def set_future(self):
-        call_args = FakeTransferFutureCallArgs(
-            fileobj=self.filename, extra_args=self.extra_args)
-        meta = FakeTransferFutureMeta(call_args=call_args)
-        return FakeTransferFuture(meta=meta)
-
-    def test_on_queued_provides_content_type(self):
-        self.subscriber.on_queued(self.future)
-        self.assertEqual(self.extra_args, {'ContentType': 'text/plain'})
-
-    def test_on_queued_does_not_provide_content_type_when_unknown(self):
-        self.filename = 'file-with-no-extension'
-        self.future = self.set_future()
-        self.subscriber.on_queued(self.future)
-        self.assertEqual(self.extra_args, {})
-
-
-class TestProvideCopyContentTypeSubscriber(
-        TestProvideUploadContentTypeSubscriber):
-    def setUp(self):
-        self.filename = 'myfile.txt'
-        self.extra_args = {}
-        self.future = self.set_future()
-        self.subscriber = ProvideCopyContentTypeSubscriber()
-
-    def set_future(self):
-        copy_source = {'Bucket': 'mybucket', 'Key': self.filename}
-        call_args = FakeTransferFutureCallArgs(
-            copy_source=copy_source, extra_args=self.extra_args)
-        meta = FakeTransferFutureMeta(call_args=call_args)
-        return FakeTransferFuture(meta=meta)
-
-
-class BaseTestWithFileCreator(unittest.TestCase):
-    def setUp(self):
-        self.file_creator = FileCreator()
-
-    def tearDown(self):
-        self.file_creator.remove_all()
-
-
-class TestProvideLastModifiedTimeSubscriber(BaseTestWithFileCreator):
-    def setUp(self):
-        super(TestProvideLastModifiedTimeSubscriber, self).setUp()
-        self.filename = self.file_creator.create_file('myfile', 'my contents')
-        self.desired_utime = datetime.datetime(
-            2016, 1, 18, 7, 0, 0, tzinfo=tzlocal())
-        self.result_queue = queue.Queue()
-        self.subscriber = ProvideLastModifiedTimeSubscriber(
-            self.desired_utime, self.result_queue)
-
-        call_args = FakeTransferFutureCallArgs(fileobj=self.filename)
-        meta = FakeTransferFutureMeta(call_args=call_args)
-        self.future = FakeTransferFuture(meta=meta)
-
-    def test_on_success_modifies_utime(self):
-        self.subscriber.on_done(self.future)
-        _, utime = get_file_stat(self.filename)
-        self.assertEqual(utime, self.desired_utime)
-
-    def test_on_success_failure_in_utime_mod_raises_warning(self):
-        self.subscriber = ProvideLastModifiedTimeSubscriber(
-            None, self.result_queue)
-        self.subscriber.on_done(self.future)
-        # Because the time to provide was None it will throw an exception
-        # which results in the a warning about the utime not being able
-        # to be set being placed in the result queue.
-        result = self.result_queue.get()
-        self.assertIsInstance(result, WarningResult)
-        self.assertIn(
-            'unable to update the last modified time', result.message)
-
-
-class TestDirectoryCreatorSubscriber(BaseTestWithFileCreator):
-    def setUp(self):
-        super(TestDirectoryCreatorSubscriber, self).setUp()
-        self.directory_to_create = os.path.join(
-            self.file_creator.rootdir, 'new-directory')
-        self.filename = os.path.join(self.directory_to_create, 'myfile')
-
-        call_args = FakeTransferFutureCallArgs(fileobj=self.filename)
-        meta = FakeTransferFutureMeta(call_args=call_args)
-        self.future = FakeTransferFuture(meta=meta)
-
-        self.subscriber = DirectoryCreatorSubscriber()
-
-    def test_on_queued_creates_directories_if_do_not_exist(self):
-        self.subscriber.on_queued(self.future)
-        self.assertTrue(os.path.exists(self.directory_to_create))
-
-    def test_on_queued_does_not_create_directories_if_exist(self):
-        os.makedirs(self.directory_to_create)
-        # This should not cause any issues if the directory already exists
-        self.subscriber.on_queued(self.future)
-        # The directory should still exist
-        self.assertTrue(os.path.exists(self.directory_to_create))
-
-    def test_on_queued_failure_propogates_create_directory_error(self):
-        # If makedirs() raises an OSError of exception, we should
-        # propogate the exception with a better worded CreateDirectoryError.
-        with mock.patch('os.makedirs') as makedirs_patch:
-            makedirs_patch.side_effect = OSError()
-            with self.assertRaises(CreateDirectoryError):
-                self.subscriber.on_queued(self.future)
-        self.assertFalse(os.path.exists(self.directory_to_create))
-
-    def test_on_queued_failure_propogates_clear_error_message(self):
-        # If makedirs() raises an OSError of exception, we should
-        # propogate the exception.
-        with mock.patch('os.makedirs') as makedirs_patch:
-            os_error = OSError()
-            os_error.errno = errno.EEXIST
-            makedirs_patch.side_effect = os_error
-            # The on_queued should not raise an error if the directory
-            # already exists
-            try:
-                self.subscriber.on_queued(self.future)
-            except Exception as e:
-                self.fail(
-                    'on_queued should not have raised an exception related '
-                    'to directory creation especially if one already existed '
-                    'but got %s' % e)
-
-
-class TestDeleteSourceObjectSubscriber(unittest.TestCase):
-    def setUp(self):
-        self.client = mock.Mock()
-        self.bucket = 'mybucket'
-        self.key = 'mykey'
-        call_args = FakeTransferFutureCallArgs(
-            bucket=self.bucket, key=self.key, extra_args={})
-        meta = FakeTransferFutureMeta(call_args=call_args)
-        self.future = mock.Mock()
-        self.future.meta = meta
-
-    def test_deletes_object(self):
-        DeleteSourceObjectSubscriber(self.client).on_done(self.future)
-        self.client.delete_object.assert_called_once_with(
-            Bucket=self.bucket, Key=self.key)
-        self.future.set_exception.assert_not_called()
-
-    def test_sets_exception_on_error(self):
-        exception = ValueError()
-        self.client.delete_object.side_effect = exception
-        DeleteSourceObjectSubscriber(self.client).on_done(self.future)
-        self.client.delete_object.assert_called_once_with(
-            Bucket=self.bucket, Key=self.key)
-        self.future.set_exception.assert_called_once_with(exception)
-
-    def test_with_request_payer(self):
-        self.future.meta.call_args.extra_args = {'RequestPayer': 'requester'}
-        DeleteSourceObjectSubscriber(self.client).on_done(self.future)
-        self.client.delete_object.assert_called_once_with(
-            Bucket=self.bucket, Key=self.key, RequestPayer='requester')
-
-
-class TestDeleteCopySourceObjectSubscriber(unittest.TestCase):
-    def setUp(self):
-        self.client = mock.Mock()
-        self.bucket = 'mybucket'
-        self.key = 'mykey'
-        copy_source = {'Bucket': self.bucket, 'Key': self.key}
-        call_args = FakeTransferFutureCallArgs(
-            copy_source=copy_source, extra_args={})
-        meta = FakeTransferFutureMeta(call_args=call_args)
-        self.future = mock.Mock()
-        self.future.meta = meta
-
-    def test_deletes_object(self):
-        DeleteCopySourceObjectSubscriber(self.client).on_done(self.future)
-        self.client.delete_object.assert_called_once_with(
-            Bucket=self.bucket, Key=self.key)
-        self.future.set_exception.assert_not_called()
-
-    def test_sets_exception_on_error(self):
-        exception = ValueError()
-        self.client.delete_object.side_effect = exception
-        DeleteCopySourceObjectSubscriber(self.client).on_done(self.future)
-        self.client.delete_object.assert_called_once_with(
-            Bucket=self.bucket, Key=self.key)
-        self.future.set_exception.assert_called_once_with(exception)
-
-    def test_with_request_payer(self):
-        self.future.meta.call_args.extra_args = {'RequestPayer': 'requester'}
-        DeleteCopySourceObjectSubscriber(self.client).on_done(self.future)
-        self.client.delete_object.assert_called_once_with(
-            Bucket=self.bucket, Key=self.key, RequestPayer='requester')
-
-
-class TestDeleteSourceFileSubscriber(unittest.TestCase):
-    def setUp(self):
-        self.tempdir = tempfile.mkdtemp()
-        self.filename = os.path.join(self.tempdir, 'myfile')
-        call_args = FakeTransferFutureCallArgs(fileobj=self.filename)
-        meta = FakeTransferFutureMeta(call_args=call_args)
-        self.future = mock.Mock()
-        self.future.meta = meta
-
-    def tearDown(self):
-        shutil.rmtree(self.tempdir)
-
-    def test_deletes_file(self):
-        with open(self.filename, 'w') as f:
-            f.write('data')
-        DeleteSourceFileSubscriber().on_done(self.future)
-        self.assertFalse(os.path.exists(self.filename))
-        self.future.set_exception.assert_not_called()
-
-    def test_sets_exception_on_error(self):
-        DeleteSourceFileSubscriber().on_done(self.future)
-        self.assertFalse(os.path.exists(self.filename))
-        call_args = self.future.set_exception.call_args[0]
-        self.assertIsInstance(call_args[0], EnvironmentError)
-
-
 class TestNonSeekableStream(unittest.TestCase):
     def test_can_make_stream_unseekable(self):
         fileobj = StringIO('foobar')
@@ -979,3 +743,141 @@ class TestNonSeekableStream(unittest.TestCase):
     def test_can_specify_amount_for_nonseekable_stream(self):
         nonseekable_fileobj = NonSeekableStream(StringIO('foobar'))
         self.assertEqual(nonseekable_fileobj.read(3), 'foo')
+
+
+class TestS3PathResolver:
+    _BASE_ACCESSPOINT_ARN = (
+        "s3://arn:aws:s3:us-west-2:123456789012:accesspoint/myaccesspoint")
+    _BASE_OUTPOST_ACCESSPOINT_ARN = (
+        "s3://arn:aws:s3-outposts:us-east-1:123456789012:outpost"
+        "/op-foo/accesspoint/myaccesspoint")
+    _BASE_ACCESSPOINT_ALIAS = "s3://myaccesspoint-foobar12345-s3alias"
+    _BASE_OUTPOST_ACCESSPOINT_ALIAS = "s3://myaccesspoint-foobar12345--op-s3"
+    _BASE_MRAP_ARN = "s3://arn:aws:s3::123456789012:accesspoint/myalias.mrap"
+
+    @pytest.mark.parametrize(
+        "path,resolved",
+        [(_BASE_ACCESSPOINT_ARN,"s3://mybucket/"),
+         (f"{_BASE_ACCESSPOINT_ARN}/","s3://mybucket/"),
+         (f"{_BASE_ACCESSPOINT_ARN}/mykey","s3://mybucket/mykey"),
+         (f"{_BASE_ACCESSPOINT_ARN}/myprefix/","s3://mybucket/myprefix/"),
+         (f"{_BASE_ACCESSPOINT_ARN}/myprefix/mykey",
+          "s3://mybucket/myprefix/mykey")]
+    )
+    def test_resolves_accesspoint_arn(
+        self, path, resolved, s3_path_resolver, s3control_client
+    ):
+        resolved_paths = s3_path_resolver.resolve_underlying_s3_paths(path)
+        assert resolved_paths == [resolved]
+        s3control_client.get_access_point.assert_called_with(
+            AccountId="123456789012",
+            Name="myaccesspoint"
+        )
+
+    @pytest.mark.parametrize(
+        "path,resolved",
+        [(_BASE_OUTPOST_ACCESSPOINT_ARN,"s3://mybucket/"),
+         (f"{_BASE_OUTPOST_ACCESSPOINT_ARN}/","s3://mybucket/"),
+         (f"{_BASE_OUTPOST_ACCESSPOINT_ARN}/mykey","s3://mybucket/mykey"),
+         (f"{_BASE_OUTPOST_ACCESSPOINT_ARN}/myprefix/",
+          "s3://mybucket/myprefix/"),
+         (f"{_BASE_OUTPOST_ACCESSPOINT_ARN}/myprefix/mykey",
+          "s3://mybucket/myprefix/mykey")]
+    )
+    def test_resolves_outpost_accesspoint_arn(
+        self, path, resolved, s3_path_resolver, s3control_client
+    ):
+        resolved_paths = s3_path_resolver.resolve_underlying_s3_paths(path)
+        assert resolved_paths == [resolved]
+        s3control_client.get_access_point.assert_called_with(
+            AccountId="123456789012",
+            Name=("arn:aws:s3-outposts:us-east-1:123456789012:outpost"
+                  "/op-foo/accesspoint/myaccesspoint")
+        )
+
+    @pytest.mark.parametrize(
+        "path,resolved",
+        [(_BASE_ACCESSPOINT_ALIAS,"s3://mybucket/"),
+         (f"{_BASE_ACCESSPOINT_ALIAS}/","s3://mybucket/"),
+         (f"{_BASE_ACCESSPOINT_ALIAS}/mykey","s3://mybucket/mykey"),
+         (f"{_BASE_ACCESSPOINT_ALIAS}/myprefix/","s3://mybucket/myprefix/"),
+         (f"{_BASE_ACCESSPOINT_ALIAS}/myprefix/mykey",
+          "s3://mybucket/myprefix/mykey")]
+    )
+    def test_resolves_accesspoint_alias(
+        self, path, resolved, s3_path_resolver, s3control_client, sts_client
+    ):
+        resolved_paths = s3_path_resolver.resolve_underlying_s3_paths(path)
+        assert resolved_paths == [resolved]
+        sts_client.get_caller_identity.assert_called_once_with()
+        s3control_client.get_access_point.assert_called_with(
+            AccountId="123456789012",
+            Name="myaccesspoint-foobar12345-s3alias"
+        )
+
+    @pytest.mark.parametrize(
+        "path",
+        [(_BASE_OUTPOST_ACCESSPOINT_ALIAS),
+         (f"{_BASE_OUTPOST_ACCESSPOINT_ALIAS}/"),
+         (f"{_BASE_OUTPOST_ACCESSPOINT_ALIAS}/mykey"),
+         (f"{_BASE_OUTPOST_ACCESSPOINT_ALIAS}/myprefix/"),
+         (f"{_BASE_OUTPOST_ACCESSPOINT_ALIAS}/myprefix/mykey")]
+    )
+    def test_outpost_accesspoint_alias_raises_exception(
+        self, path, s3_path_resolver
+    ):
+        with pytest.raises(ParamValidationError) as e:
+            s3_path_resolver.resolve_underlying_s3_paths(path)
+        assert "Can't resolve underlying bucket name" in str(e.value)
+
+    @pytest.mark.parametrize(
+        "path,resolved",
+        [(_BASE_MRAP_ARN,"s3://mybucket/"),
+         (f"{_BASE_MRAP_ARN}/","s3://mybucket/"),
+         (f"{_BASE_MRAP_ARN}/mykey","s3://mybucket/mykey"),
+         (f"{_BASE_MRAP_ARN}/myprefix/","s3://mybucket/myprefix/"),
+         (f"{_BASE_MRAP_ARN}/myprefix/mykey","s3://mybucket/myprefix/mykey")]
+    )
+    def test_resolves_mrap_arn(
+        self, path, resolved, s3_path_resolver, s3control_client
+    ):
+        resolved_paths = s3_path_resolver.resolve_underlying_s3_paths(path)
+        assert resolved_paths == [resolved]
+        s3control_client.list_multi_region_access_points.assert_called_with(
+            AccountId="123456789012"
+        )
+
+    @pytest.mark.parametrize(
+            "path,resolved,name",
+            [(f"{_BASE_ACCESSPOINT_ARN}-s3alias/mykey","s3://mybucket/mykey",
+              "myaccesspoint-s3alias"),
+             (f"{_BASE_OUTPOST_ACCESSPOINT_ARN}--op-s3/mykey",
+              "s3://mybucket/mykey",
+              f"{_BASE_OUTPOST_ACCESSPOINT_ARN[5:]}--op-s3")]
+    )
+    def test_alias_suffixes_dont_match_accesspoint_arns(
+        self, path, resolved, name, s3_path_resolver, s3control_client
+    ):
+        resolved_paths = s3_path_resolver.resolve_underlying_s3_paths(path)
+        assert resolved_paths == [resolved]
+        s3control_client.get_access_point.assert_called_with(
+            AccountId="123456789012",
+            Name=name
+        )
+
+    @pytest.mark.parametrize(
+            "path,expected_has_underlying_s3_path",
+            [(_BASE_ACCESSPOINT_ARN,True),
+             (f"{_BASE_ACCESSPOINT_ARN}/mykey",True),
+             (f"{_BASE_ACCESSPOINT_ARN}/myprefix/mykey",True),
+             (_BASE_ACCESSPOINT_ALIAS,True),
+             (_BASE_OUTPOST_ACCESSPOINT_ARN,True),
+             (_BASE_OUTPOST_ACCESSPOINT_ALIAS,True),
+             (_BASE_MRAP_ARN,True),
+             ("s3://mybucket/",False),
+             ("s3://mybucket/mykey",False),
+             ("s3://mybucket/myprefix/mykey",False)]
+    )
+    def test_has_underlying_s3_path(self, path, expected_has_underlying_s3_path):
+        has_underlying_s3_path = S3PathResolver.has_underlying_s3_path(path)
+        assert has_underlying_s3_path == expected_has_underlying_s3_path
