@@ -21,7 +21,7 @@ import shlex
 import botocore
 import botocore.model
 import botocore.session as session
-from botocore.exceptions import ConnectionClosedError
+from botocore.exceptions import ConnectionClosedError, MetadataRetrievalError
 from awscli.clidriver import create_clidriver
 from awscli.testutils import unittest, skip_if_windows, mock
 from awscli.compat import is_windows
@@ -29,7 +29,9 @@ from awscli.utils import (
     split_on_commas, ignore_ctrl_c, find_service_and_method_in_event_name,
     is_document_type, is_document_type_container, is_streaming_blob_type,
     is_tagged_union_type, operation_uses_document_types, dump_yaml_to_str,
-    ShapeWalker, ShapeRecordingVisitor, OutputStreamFactory, LazyPager
+    ShapeWalker, ShapeRecordingVisitor, OutputStreamFactory, LazyPager,
+    add_command_lineage_to_user_agent_extra,
+    add_metadata_component_to_user_agent_extra,
 )
 from awscli.utils import InstanceMetadataRegionFetcher
 from awscli.utils import IMDSRegionProvider
@@ -316,7 +318,7 @@ class TestOutputStreamFactory(unittest.TestCase):
         self.assertEqual(self.popen.call_count, 0)
 
 
-class TestInstanceMetadataRegionFetcher(unittest.TestCase):
+class BaseIMDSRegionTest(unittest.TestCase):
     def setUp(self):
         urllib3_session_send = 'botocore.httpsession.URLLib3Session.send'
         self._urllib3_patch = mock.patch(urllib3_session_send)
@@ -324,9 +326,13 @@ class TestInstanceMetadataRegionFetcher(unittest.TestCase):
         self._imds_responses = []
         self._send.side_effect = self.get_imds_response
         self._region = 'us-mars-1a'
+        self.environ = {}
+        self.environ_patch = mock.patch('os.environ', self.environ)
+        self.environ_patch.start()
 
     def tearDown(self):
         self._urllib3_patch.stop()
+        self.environ_patch.stop()
 
     def add_imds_response(self, body, status_code=200):
         response = botocore.awsrequest.AWSResponse(
@@ -354,6 +360,8 @@ class TestInstanceMetadataRegionFetcher(unittest.TestCase):
             raise response
         return response
 
+
+class TestInstanceMetadataRegionFetcher(BaseIMDSRegionTest):
     def test_disabled_by_environment(self):
         env = {'AWS_EC2_METADATA_DISABLED': 'true'}
         fetcher = InstanceMetadataRegionFetcher(env=env)
@@ -454,15 +462,7 @@ class TestInstanceMetadataRegionFetcher(unittest.TestCase):
         self.assertEqual(result, None)
 
 
-class TestIMDSRegionProvider(unittest.TestCase):
-    def setUp(self):
-        self.environ = {}
-        self.environ_patch = mock.patch('os.environ', self.environ)
-        self.environ_patch.start()
-
-    def tearDown(self):
-        self.environ_patch.stop()
-
+class TestIMDSRegionProvider(BaseIMDSRegionTest):
     def assert_does_provide_expected_value(self, fetcher_region=None,
                                            expected_result=None,):
         fake_session = mock.Mock(spec=session.Session)
@@ -484,74 +484,89 @@ class TestIMDSRegionProvider(unittest.TestCase):
             expected_result=None,
         )
 
-    @mock.patch('botocore.httpsession.URLLib3Session.send')
-    def test_use_truncated_user_agent(self, send):
+    def test_use_truncated_user_agent(self):
         driver = create_clidriver()
         driver.session.user_agent_version = '3.0'
+        self.add_imds_token_response()
+        self.add_get_region_imds_response()
         provider = IMDSRegionProvider(driver.session)
         provider.provide()
-        args, _ = send.call_args
+        args, _ = self._send.call_args
         self.assertEqual(args[0].headers['User-Agent'], 'aws-cli/3.0')
 
-    @mock.patch('botocore.httpsession.URLLib3Session.send')
-    def test_can_use_ipv6(self, send):
+    def test_can_use_ipv6(self):
         driver = create_clidriver()
         driver.session.set_config_variable('imds_use_ipv6', True)
+        self.add_imds_token_response()
+        self.add_get_region_imds_response()
         provider = IMDSRegionProvider(driver.session)
         provider.provide()
-        args, _ = send.call_args
+        args, _ = self._send.call_args
         self.assertIn('[fd00:ec2::254]', args[0].url)
 
-    @mock.patch('botocore.httpsession.URLLib3Session.send')
-    def test_use_ipv4_by_default(self, send):
+    def test_use_ipv4_by_default(self):
         driver = create_clidriver()
+        self.add_imds_token_response()
+        self.add_get_region_imds_response()
         provider = IMDSRegionProvider(driver.session)
         provider.provide()
-        args, _ = send.call_args
+        args, _ = self._send.call_args
         self.assertIn('169.254.169.254', args[0].url)
 
-    @mock.patch('botocore.httpsession.URLLib3Session.send')
-    def test_can_set_imds_endpoint_mode_to_ipv4(self, send):
+    def test_can_set_imds_endpoint_mode_to_ipv4(self):
         driver = create_clidriver()
         driver.session.set_config_variable(
             'ec2_metadata_service_endpoint_mode', 'ipv4')
+        self.add_imds_token_response()
+        self.add_get_region_imds_response()
         provider = IMDSRegionProvider(driver.session)
         provider.provide()
-        args, _ = send.call_args
+        args, _ = self._send.call_args
         self.assertIn('169.254.169.254', args[0].url)
 
-    @mock.patch('botocore.httpsession.URLLib3Session.send')
-    def test_can_set_imds_endpoint_mode_to_ipv6(self, send):
+    def test_can_set_imds_endpoint_mode_to_ipv6(self):
         driver = create_clidriver()
         driver.session.set_config_variable(
             'ec2_metadata_service_endpoint_mode', 'ipv6')
+        self.add_imds_token_response()
+        self.add_get_region_imds_response()
         provider = IMDSRegionProvider(driver.session)
         provider.provide()
-        args, _ = send.call_args
+        args, _ = self._send.call_args
         self.assertIn('[fd00:ec2::254]', args[0].url)
 
-    @mock.patch('botocore.httpsession.URLLib3Session.send')
-    def test_can_set_imds_service_endpoint(self, send):
+    def test_can_set_imds_service_endpoint(self):
         driver = create_clidriver()
         driver.session.set_config_variable(
             'ec2_metadata_service_endpoint', 'http://myendpoint/')
+        self.add_imds_token_response()
+        self.add_get_region_imds_response()
         provider = IMDSRegionProvider(driver.session)
         provider.provide()
-        args, _ = send.call_args
+        args, _ = self._send.call_args
         self.assertIn('http://myendpoint/', args[0].url)
 
-    @mock.patch('botocore.httpsession.URLLib3Session.send')
-    def test_imds_service_endpoint_overrides_ipv6_endpoint(self, send):
+    def test_imds_service_endpoint_overrides_ipv6_endpoint(self):
         driver = create_clidriver()
         driver.session.set_config_variable(
             'ec2_metadata_service_endpoint_mode', 'ipv6')
         driver.session.set_config_variable(
             'ec2_metadata_service_endpoint', 'http://myendpoint/')
+        self.add_imds_token_response()
+        self.add_get_region_imds_response()
         provider = IMDSRegionProvider(driver.session)
         provider.provide()
-        args, _ = send.call_args
+        args, _ = self._send.call_args
         self.assertIn('http://myendpoint/', args[0].url)
 
+    def test_disable_ec2_metadata_v1(self):
+        driver = create_clidriver()
+        driver.session.set_config_variable('ec2_metadata_v1_disabled', True)
+        self.add_imds_response(b'', 404)
+        provider = IMDSRegionProvider(driver.session)
+        with self.assertRaises(MetadataRetrievalError):
+            provider.provide()
+
 
 class TestLazyPager(unittest.TestCase):
     def setUp(self):
@@ -824,3 +839,26 @@ class TestTaggedUnion:
 
     def test_shape_is_not_tagged_union(self, argument_model):
         assert not is_tagged_union_type(argument_model)
+
+
+@pytest.fixture
+def test_session():
+    test_session = mock.Mock(session.Session)
+    test_session.user_agent_extra = ''
+    return test_session
+
+
+def test_add_metadata_component_to_user_agent_extra(test_session):
+    add_metadata_component_to_user_agent_extra(test_session, 'name', 'value')
+    assert test_session.user_agent_extra == 'md/name#value'
+
+
+def test_add_command_lineage_to_user_agent_extra(test_session):
+    add_command_lineage_to_user_agent_extra(test_session, ['a', 'b', 'c'])
+    assert test_session.user_agent_extra == 'md/command#a.b.c'
+
+
+def test_no_add_command_lineage_to_user_agent_extra_with_existing_command_lineage(test_session):
+    test_session.user_agent_extra = 'md/command#a.b.c'
+    add_command_lineage_to_user_agent_extra(test_session, ['a', 'b', 'c', 'd'])
+    assert test_session.user_agent_extra == 'md/command#a.b.c'
